/*
 * $Id: cia.h,v 1.3 1995/06/28 19:40:56 jopi Exp jopi $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 * CIA 6526 and VIA 6522 register number declarations.
 * Interrupt signals are defined in vmachine.h.
 *
 * Written by
 *    Jouko Valta  (jopi@stekt.oulu.fi)
 *
 *
 * $Log: cia.h,v $
 * Revision 1.3  1995/06/28  19:40:56  jopi
 * Never count on the documents...
 *
 * Revision 1.2  1994/12/12  16:59:44  jopi
 * Defines for registers
 *
 * Revision 1.1  1994/08/10  17:46:41  jopi
 * Initial revision
 *
 *
 */

#ifndef X64_CIA_H
#define X64_CIA_H


#ifdef VIA6522

/*
 * MOS 6522 registers
 */
#define P_PORT_B	0  /* Port B */
#define P_PORT_A	1  /* Port A */
#define P_DDIR_B	2  /* Data direction register for port B */
#define P_DDIR_A	3  /* Data direction register for port A */

#define P_TIMER_AL	4  /* Timer A count low */
#define P_TIMER_AH	5  /* Timer A count high */
#define P_TIMER_L_AL	6  /* Timer A latch low */
#define P_TIMER_L_AH	7  /* Timer A latch high */
#define P_TIMER_BL	8  /* Timer B latch/count low */
#define P_TIMER_BH	9  /* Timer B latch/count high */

#define P_SERIAL	10 /* Serial port shift register */
#define P_AUX		11 /* Auxiliary control register */
#define P_PERIPH	12 /* Peripheral control register */

#define P_INTFLAG	13 /* Interrupt flag register */
#define P_INTENAB	14 /* Interrupt control register */
#define P_PORT_A_NHS	15 /* Port A with no handshake */
#else

/*
 * MOS 6526 registers
 */
#define P_PORT_A	0  /* Port A */
#define P_PORT_B	1  /* Port B */
#define P_DDIR_A	2  /* Data direction register for port A */
#define P_DDIR_B	3  /* Data direction register for port B */

#define P_TIMER_AL	4  /* Timer A low */
#define P_TIMER_AH	5  /* Timer A high */
#define P_TIMER_BL	6  /* Timer B low */
#define P_TIMER_BH	7  /* Timer B high */

#define P_TOD_TEN	8  /* Time Of Day clock 1/10 s */
#define P_TOD_SEC	9  /* Time Of Day clock sec */
#define P_TOD_MIN	10 /* Time Of Day clock min */
#define P_TOD_HR	11 /* Time Of Day clock hour */

#define P_SERIAL	12 /* Serial port shift register */
#define P_INTFLAG	13 /* Interrupt flag register */
#define P_INTENAB	13 /* Interrupt control register */
#define P_CONT_A	14 /* Control register A */
#define P_CONT_B	15 /* Control register B */
#endif


/*
 * Interrupt Masks
 */

#ifdef VIA6522
/* MOS 6522 */
#define IM_SET	0x80	/* Control Bit */
#define IM_TA	64	/* Timer A underflow */
#define IM_TB	32	/* Timer B underflow */
#define IM_CB1	16	/* Handshake */
#define IM_CB2	8	/* Handshake */
#define IM_SP	4	/* Shift Register completion */
#define IM_CA1	2	/* Handshake */
#define IM_CA2	1	/* Handshake */
#else
/* MOS 6526 */
#define IM_SET	0x80	/* Control Bit */
#define IM_TA	1	/* Timer A underflow */
#define IM_TB	2	/* Timer B underflow */
#define IM_TOD	4	/* TOD Clock Alarm */
#define IM_SP	8	/* Shift Register completion */
#define IM_FLG	16	/* Handshake */
#endif


#endif  /* X64_CIA_H */
