/*
 * $Id: config.h,v 1.11 1995/11/07 16:51:00 jopi Exp jopi $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 * Patches by
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 */


/* ------------------------------------------------------------------------- *
 *
 * VICE Configuration Set Up
 * Recommendations for each define are shown after them, so
 * that you can restore the settings.
 *
 * Use 'xmkmf' to update the Makefile.
 *
 *  Tips on Installation:
 *  For DEC, if necessary, enable XCONTROL, and DECMAP
 *  For DEC, disable XPM
 *
 * ------------------------------------------------------------------------- */

/*
 * Interfaces, Windows
 */


#if !defined (_CONFIG_H)
#define _CONFIG_H

/*
 * Don't try this at home...
 */

#if (!defined(PET) && !defined(VIC20))
/*#define NEW_VIC*/		/* no */
#endif

#define VIC_II			/* yes */


/*
 * X11 front-end for the debugger
 *
 * 'XCONTROL' selects the old Control Window from X64-0.2.2
 *  to use in case the new one is indigestible.
 *  (not recommended if the new 'xdebug.c' compiles.)
 */

/*#define XCONTROL*/		/* no */

#ifndef XCONTROL
#define XDEBUGGER		/* yes */
#endif


/*
 * If you want to use command line editor with monitor, define following
 * It uses termios interface for a tty by default, uncomment next line
 * if you only have termio
 */

/*#undef EDIT*/
#define EDIT			/* yes */


/*
 * Which way to copy files ?
 * Since C1541 disk drive copies files to the first filename, whereas
 * they normally go to the last file, one can change the 'c1541' copy
 * syntax via this option.
 *   UNIX:  copy    oldname [oldname2 ...] newname
 *   1541:  copy    newname oldname [oldname2 ...]
 */

/*#define COPY_TO_LEFT*/	/* no */


/*
 * Do you have systeminfo.h and sysinfo(2) ?
 */

#if (defined(SUN))
#define SYSINFO			/* yes */
#endif


/*
 * Do you have the Xpm library ?
 */

#define XPM			/* yes */


/* ------------------------------------------------------------------------- */


/*
 * Window Sizes and refresh rate
 *
 * Select the desired size and refresh rate for the video emulation windows.
 * You can choose whether you want double/triple size for width and/or height
 * and the maximum refresh rate for each video chip emulated.
 * Setting REFRESH_RATE to <x> means that 1/<x> of the real refreshes are done
 * i.e. the higher REFRESH_RATE is the faster (and less accurate) the
 * emulation is.
 * You will be allowed to change the REFRESH_RATE value at runtime or with a
 * command-line option.
 */


/*
 * First of all, specify whether you want PAL or NTSC.
 */

#define PAL			/* yes -- we come from Europe :) */
/*#define NTSC*/		/* no */


/**** VIC20 (xvic) settings ****/

/* (QUAD_X is available for VIC20 only) */

#if defined( VIC20 )

#define REFRESH_RATE 4

#define DOUBLE_X 		/* yes */

/*#define TRIPLE_X*/		/* no */
/*#define DOUBLE_Y*/		/* no */

/*#define QUAD_X*/		/* no */
/*#define TRIPLE_Y*/		/* no */


#elif defined( C128 )

/**** C128 (x128) settings ****/

/* 80 column (VDC) window */
#define VDC_REFRESH_RATE 4
/* 40 column (VIC) window */
#define VIC_REFRESH_RATE 4

/* (the other VIC and VDC settings are separate) */

#if defined( VIC )

/* x128/VIC settings */

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */    	/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */		/* no */

#else

/* x128/VDC settings */

/* #define DOUBLE_X */       	/* no */
/* #define TRIPLE_X */		/* no */
#define DOUBLE_Y		/* yes */
/* #define VDC_TRIPLE_Y */      /* no */

#endif

#elif defined( PET )

/**** PET (xpet) settings ****/

#define REFRESH_RATE 4

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */		/* no */
#define DOUBLE_Y		/* no */
/* #define TRIPLE_Y */		/* no */

#else

/**** C64 (x64) settings ****/

#define REFRESH_RATE 4

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */		/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */		/* no */

#endif


/* ------------------------------------------------------------------------- */

/*
 * Keyboard
 */


/*
 * Set up Keyboard Input Buffer in X-version
 *
 * Some keyboards do not generate KeyRelease Event, some have
 * different layout. Try which settings are best for your keyboard.
 */

/*#define AUTOREPEAT*/	/* no */	/* operate without keyrelease events */
/*#define US_KBD*/	/* US only */	/* swap asciicircum with asciitilde */


/*
 * Host keyboard repeat treshold (ms). This filters out any excessive
 * key events, so it must be longer than actual autorepeat interval.
 */

#ifndef C128
#define REPEAT_DELAY	50	/* near 50 */
#else
#define REPEAT_DELAY	100
#endif


/*
 * Some keys only for DEC with additonal Editing Keypad
 * Only enable this if you have the Editing Keymap.
 */

/*#define DECMAP*/		/* no */


/* ------------------------------------------------------------------------- */

/*
 * CPU
 */

/*
 * "Hacked" and faster CPU emulation
 * For CPU debugging, NORMAL (preferred) or HACK2 should be used.
 */

#define HACK2			/* yes */
/*#define HACK*/		/* no */
/*#define NORMAL*/		/* no */


/*
 * If you do NOT want to emulate undocumented commands, uncomment
 * the line below, and also select "NORMAL" CPU emulation.
 */

/*#define NO_UNDOC_CMDS*/	/* no */


/*
 *  LED Bar - a simple toy.
 *  This only works with ophack2, and makes it slower than usual.
 */

#ifndef DEBUG
/*#define LED_ARRAY*/		/* no */
#endif


/*
 * Install Zilog Z80B CPU on the C128 ?
 */

#if (defined(DO_CONFIG) || defined(C128))

/* #define HAS_ZILOG_Z80 */		/* no */

#endif


/* ------------------------------------------------------------------------- */


/*
 * Joystick (needs the joystick module for Linux)
 * This won't affect the `virtual' joystick emulation via Numpad.
 */

/*#define JOYSTICK*/		/* Linux PC with joystick only */


/*
 * Light Pen (VIC-II, VIC-IIe, VDC)
 */

/*#define LIGHTPEN*/		/* yes */


/*
 * Peripherals
 */

/* gone...
#ifdef PET
#define NO_SERIAL
#endif
*/

/*
 * Printer
 */

/* Select Default Character Set */

#define PRN_CS_DEFAULT  0	/* 0 */


#if 1  /* European A4 Papersize */
#define PS_WIDTH	595
#define PS_HEIGHT	842

#else  /* American Letter Papersize */
#define PS_WIDTH	612
#define PS_HEIGHT	792
#endif


/* ------------------------------------------------------------------------- */


/*
 * Video emulation
 */


/*
 * Do you want the VIC-II sprite-sprite and sprite-background collision
 * registers to be emulated by default?
 */

/*#define CHECK_SS_COLL*/	/* no */
/*#define CHECK_SB_COLL*/	/* no */


/*
 * Enable C128 VDC by default?
 */

#define USE_VDC			/* no */


/*
 * PET Screen width. It can be either 40 or 80 Columns.
 */

#define PET_COLS	80


/* ------------------------------------------------------------------------- */


/*
 * Speed
 *
 * If you have a fast machine and you want to slow the emulation down to 
 * a fixed speed index, simply fill in the appropriate values (percentage
 * values referred to the real machine).
 * If you specify a value of 0, the emulators will try to run as fast as
 * possible.
 * You will still be allowed to change these values at runtime or on the
 * command line, though.
 */

#define PET_CUSTOM_SPEED	100	/* 0 */
#define C64_CUSTOM_SPEED	100	/* 0 */
#define VIC20_CUSTOM_SPEED	100	/* 0 */
#define C128_CUSTOM_SPEED	100	/* 0 */


/* ------------------------------------------------------------------------- */


/*
 * Miscellaneous
 */


/*
 * Inline functions increase speed very much, but if your compiler does not
 * support them or you wish to obtain a smaller executable uncomment this
 * line.
 */

#define NO_INLINE		/* no */


/*
 * 40xx or 80xx PET ?
 */

#define BASIC4


/*
 * Shall Virtual RAM Expander be installed ?
 * Sizes available are 128, 256 and 512 KB
 * (C128 internal RAM may be expanded in vmachine.h)
 */

#ifndef VIC20
#ifndef PET

#define REU			/* yes */
#define REUSIZE 256

#endif
#endif


/*
 * Utility to patch between ROM revisions.
 * Supports 0, 3, 67 and 100 for the C64, but user can define more.
 */

#ifndef VIC20
#ifndef PET

#define PATCH_ROM		/* yes */
#endif
#endif


/*
 * Emulator Identification Utility
 */

#define EMULATOR_ID		/* yes */


/*
 * Trap idle
 */

/*#define IDLE_TRAP*/		/* no */


/*
 * Experimental timer code
 */

#define NEW_TIMER_CODE		/* yes */


/* ------------------------------------------------------------------------- */

/* DO NOT change this... */
#if (!defined(C128) && !defined(VIC20) &&!defined(PET))
#define CBM64
#endif

/*
 * Define a few default filenames
 */

#ifndef PROJECTDIR
#ifdef C128
#define PROJECTDIR	"C128"
#endif

#ifdef CBM64
#define PROJECTDIR	"C64"
#endif

#ifdef VIC20
#define PROJECTDIR	"VIC20"
#endif

#ifdef PET
#ifdef PET8__
#define PROJECTDIR	"PET8"
#else
#define PROJECTDIR	"PET"
#endif
#endif

#endif  /* PROJECTDIR */


#ifndef RAMNAME
#define RAMNAME		"ram"
#endif

#ifndef BASICNAME
#define BASICNAME	"basic"
#endif

#ifndef KERNELNAME
#define KERNELNAME	"kernel"
#endif

#ifndef CHARGENNAME
#define CHARGENNAME	"chargen"
#endif


#ifndef REUNAME
#define REUNAME		"reu"
#endif

#ifndef EXROMNAME
#define EXROMNAME	"exrom"
#endif

#ifndef BIOSROM_NAME
#define BIOSROM_NAME  "Z80_BIOS"
#endif


/* ------------------------------------------------------------------------- */


/*
 * --DEBUGGING--
 *
 * We all hope you won't need these ones... but you never know :)
 * Take care, since some of these options print a really _huge_ amount of
 * stuff!
 */


/*
 * In case of problems with the keyboard...
 */

/* #define KBDBUG */	/* no */	/* full */
/* #define KBDBUG2 */	/* no */	/* keypress stack only */


/*
 * In case of problems with the disk drives...
 */

/* #define DEBUG_DRIVE */		/* no */
/* #define DEBUG_FSDRIVE */	/* no */

/* #define DEBUG_SERIAL */	/* no */	/* Serial control */
/* #define DEBUG_SERIAL_1 */	/* no */	/* Serial data flow */


/*
 * In case of problems with the video emulations...
 */

/* #define VIC_II_VMODE_DEBUG */	/* no */
/* #define VIC_II_RASTER_DEBUG */	/* no */
/* #define VIC_II_INTERRUPT_DEBUG */	/* no */

/* #define VDC_REG_DEBUG */		/* no */
/* #define VDC_VMODE_DEBUG */		/* no */

/* #define VIC_VMODE_DEBUG */		/* no */
/* #define VIC_RASTER_DEBUG */		/* no */


/*
 * In case of problems with the CIAs...
 */

/* #define CIA1_TIMER_DEBUG */          /* no */
/* #define CIA2_TIMER_DEBUG */          /* no */


/*
 * In case you have problems with the Z80 emulation...
 */

#if defined (HAS_ZILOG_Z80)
/* #define DEBUG_Z80 */		        /* no */
#endif


/*
 * Uncommenting the following line causes some debug information to be printed
 * and with 'NORMAL', execution of different 6510 intructions can be counted.
 */

/* #define DEBUG */		        /* no */


/* ------------------------------------------------------------------------- */

#ifdef __hpux
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#ifndef _INCLUDE_POSIX_SOURCE
#define _INCLUDE_POSIX_SOURCE
#endif
#endif  /* __hpux */


#endif  /* _CONFIG_H */
