/*
 * $Id: sprcoll.c,v 1.1 1996/04/01 08:25:07 jopi Exp jopi $
 *
 * This file contains the code required for sprite-sprite and sprite-background
 * collision register emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: sprcoll.c,v $
 * Revision 1.1  1996/04/01  08:25:07  jopi
 * Initial revision
 *
 *
 */

#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>

#include "sprcoll.h"

/*

   some functions for debugging pourposes only...

 */

void
print_binary (BYTE value)
{
  BYTE msk;

  for (msk = 1 << 7; msk; msk >>= 1)
    putchar (value & msk ? 'H' : '-');
}

void
print_huge_mask (sprmask_t * mask, int size)
{
  int i;

  for (i = 0; i < size; i++)
    print_binary (mask[i]);
}


/* ------------------------------------------------------------------------------------------ */

void
build_mask (BYTE * src, int sprsize, sprmask_t * sprmask_return, int sh_right)
{
  int i;
  BYTE tempmask, savemask;
  int sh_left = 8 - sh_right;

  savemask = 0xff >> sh_left;
  tempmask = 0;
  for (i = 0; i < sprsize; i++, src++, sprmask_return++)
    {
      *sprmask_return = tempmask | (*src >> sh_right);
      tempmask = (*src & savemask) << sh_left;
    }
  *sprmask_return = tempmask;
}

void
double_mask (sprmask_t * sprmask, int size)
{
  int i, j;
  sprmask_t src[MASK_SIZE], srcmsk, destmsk;

  for (i = 0; i < size; i++)
    src[i] = sprmask[i];

  for (j = -1, i = 0; i < size; i++)
    {
      sprmask[++j] = 0;
      for (destmsk = 3 << 6, srcmsk = 1 << 7; srcmsk > 8; srcmsk >>= 1, destmsk >>= 2)
	if (src[i] & srcmsk)
	  sprmask[j] |= destmsk;
      sprmask[++j] = 0;
      for (destmsk = 3 << 6; srcmsk; srcmsk >>= 1, destmsk >>= 2)
	if (src[i] & srcmsk)
	  sprmask[j] |= destmsk;
    }
}

/*

   convert_multi () transforms the mask so that each couple of bits is turned into a couple
   of '1' bits when it represents a coloured pixel in multicolor mode, allowing correct
   sprite-sprite collision with multicolor sprites, too.

 */

void
convert_multi (sprmask_t * dest, BYTE * src, int size)
{
  int i, msk;

  for (i = 0; i < size; i++, dest++, src++)
    {
      *dest = 0;
      for (msk = 3 << 6; msk; msk >>= 2)
	if (*src & msk)
	  *dest |= msk;
    }
}

/*

   build_sprite_masks () prepares the masks for all the sprites on the current raster line
   for the VideoStruct specified. It also returns the sizes (number of masks required) of
   all the sprites and the starting and ending columns for collision checking.

   The starting and ending columns refer to the current position of the drawable screen
   and their values can be out of the normal screen range (eg they can be < 0). Dividing
   the sprite masks this way allows faster collision checking (though this is probably a
   quite weird way to do that).

   Returns the number of masks built.

 */

int
build_sprite_masks (VideoStruct * v, sprmask_struct * sprmasks_return, int line)
{
  int i, nbuilt = 1;
  LineStruct *l = &v->lineinfo[line];
  SpriteLineStruct *s = l->sprites;

  for (i = 0; i < l->numsprites; i++, s++, sprmasks_return++)
    if ((sprmasks_return->viewable = s->viewable))
      {
	int xrelpos;
	sprmask_t tmpmsk[6];
	if (s->multicolor)
	  convert_multi (tmpmsk, s->data, 3);
	else
	  {
	    int k;
	    for (k = 0; k < 3; k++)
	      tmpmsk[k] = s->data[k];
	  }
	if (s->xexp)
	  double_mask (tmpmsk, 3);
	nbuilt++;
	xrelpos = s->x - (v->bordwidth + l->xsmooth);
	if (xrelpos > 0)
	  sprmasks_return->startcol = xrelpos >> 3;
	else
	  sprmasks_return->startcol = -((-xrelpos) >> 3) - 1;
	sprmasks_return->size = s->xexp ? 8 : 4;
	sprmasks_return->endcol = sprmasks_return->startcol + sprmasks_return->size - 1;
	build_mask (tmpmsk, s->xexp ? 6 : 3, sprmasks_return->mask,
		    xrelpos < 0 ? ((-xrelpos) % 8) : (xrelpos % 8));
      }
#if 0				/* paranoia debugging */
  if (nbuilt)
    {
      printf ("build_sprite_masks (): built %d mask(s) out of %d sprites:\n", nbuilt, v->numsprites);
      for (i = l->numsprites - 1; i >= 0; i--, sprmasks_return--)
	{
	  printf ("\tsprite %d: ", i);
	  print_huge_mask (sprmasks_return->mask, MASK_SIZE);
	  printf (" - offset %d\n", sprmasks_return->startcol);
	}
    }
#endif
  return nbuilt;
}

/*

   check_ss_collisions () checks the sprite-sprite collisions.

 */

sprcollmask_t
check_ss_collisions (sprmask_struct * s)
{
  int i, j, k, msk1, msk2, l;
  sprmask_t *p1, *p2;
  sprcollmask_t mask_return = 0;

  for (msk1 = 1, i = 0; i < NUM_SPRITES - 1; i++, msk1 <<= 1)
    for (msk2 = msk1 << 1, j = i + 1; j < NUM_SPRITES; j++, msk2 <<= 1)
      if (s[j].viewable && s[i].viewable)
	{
	  if (s[j].startcol <= s[i].endcol && s[i].endcol <= s[j].endcol)
	    {
	      l = s[i].endcol - s[j].startcol + 1;
	      p1 = s[j].mask;
	      p2 = s[i].mask + s[j].startcol - s[i].startcol;
	    }
	  else if (s[j].startcol <= s[i].startcol && s[i].startcol <= s[j].endcol)
	    {
	      l = s[j].endcol - s[i].startcol + 1;
	      p1 = s[i].mask;
	      p2 = s[j].mask + s[i].startcol - s[j].startcol;
	    }
	  else
	    break;
	  for (k = 0; k < l; k++, p1++, p2++)
	    if (*p1 & *p2)
	      {
		mask_return |= msk1 | msk2;
		break;
	      }
	}
  return mask_return;
}

/*

   chk_sb_collisions () checks the sprite-to-background collisions.

 */


sprcollmask_t
check_sb_collisions (VideoStruct * v, sprmask_struct * sprmasks, int line)
{
  int i, n, s, j, x;
  sprcollmask_t msk, mask_return = 0;
  BYTE *data;

  if ((line < (v->bordheight + v->ysmooth)) ||
      (line >= (v->bordheight + v->ypix - v->brdgrow_yl)))
    return 0;

  data = v->lineinfo[line].fgdata;
  for (i = 0, msk = 1; i < 8; i++, sprmasks++, msk <<= 1)
    if (sprmasks->viewable && sprmasks->endcol >= 0 && sprmasks->startcol < v->tcols)
      {
	n = sprmasks->size;
	s = 0;
	if (sprmasks->startcol < 0)
	  {
	    n += sprmasks->startcol;
	    s -= sprmasks->startcol;
	    x = 0;
	  }
	else
	  x = sprmasks->startcol;
	if (sprmasks->endcol >= v->tcols)
	  n -= sprmasks->endcol - (v->tcols - 1);
	for (j = 0; j < n; j++, s++, x++)
	  if (data[x] & sprmasks->mask[s])
	    {
	      mask_return |= msk;
	      break;
	    }
      }
  return mask_return;
}
