/*
 * $Id: sprcoll.h,v 1.1 1996/04/01 08:25:07 jopi Exp jopi $
 *
 * This file contains the code required for sprite-sprite and sprite-background
 * collision register emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: sprcoll.h,v $
 * Revision 1.1  1996/04/01  08:25:07  jopi
 * Initial revision
 *
 *
 */

#if !defined (_SPRCOLL_H)

#include "raster.h"

#define _SPRCOLL_H
#define BYTE unsigned char

#define NUM_SPRITES	8
#define SPR_SIZE	3
#define MASK_SIZE	(SPR_SIZE + 1) * 2

typedef BYTE sprmask_t;		/* mask of the sprite image */
typedef BYTE sprcollmask_t;	/* mask for colliding sprites (1 = sprite is colliding) */

typedef struct
  {
    sprmask_t mask[MASK_SIZE];	/* sprite mask */
    int size;			/* size (number of masks) */
    int startcol;		/* first column for collision check (can be <0 or >ncols) */
    int endcol;			/* last column for collision check (can be <0 or >ncols) */
    int viewable;
  }
sprmask_struct;

void build_mask (BYTE * src, int sprsize, sprmask_t * sprmask_return, int sh_right);
void double_mask (sprmask_t * sprmask, int size);
void convert_multi (sprmask_t *dest, BYTE *src, int size);
int build_sprite_masks (VideoStruct * v, sprmask_struct * sprmasks_return, int line);
sprcollmask_t check_ss_collisions (sprmask_struct * s);
sprcollmask_t check_sb_collisions (VideoStruct * v, sprmask_struct * sprmasks, int line);

/* debugging-only functions */

void print_binary (BYTE value);
void print_huge_mask (sprmask_t *mask, int size);

#endif /* !defined (SPRCOLL_H) */
