/*
 * $Id: vdc.c,v 1.1 1996/04/01 05:48:13 ettore Exp $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * VDC video chip emulation.
 * Only 80x25 text mode is supported now.
 *
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vdc.c,v $
 * Revision 1.1  1996/04/01 05:48:13  ettore
 * Initial revision
 *
 */


#define _VDC_C

#include "extern.h"
#include "config.h"
#include "vdc.h"
#include "raster.c"
#include "macro.h"
#include "vmachine.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

/* #define VDC_REG_DEBUG */
/* #define VDC_VMODE_DEBUG */


/* global */

static BYTE vdc[37];
static BYTE vdcRAM[65535];	/* possible values: 16K, 2*32K, or 64K */

VideoStruct *v_vdc;


/* functions */

void vdc_set_vmode (Window win, int mode);
int vdc_init (void);
void store_vdc (BYTE reg, BYTE value);
BYTE read_vdc (BYTE reg);


/*

   store_vdc(): stores a value in a VDC register.

 */

void
store_vdc (BYTE reg, BYTE value)
{
#if defined( VDC_REG_DEBUG )
  printf ("store_vdc reg[ %d ] = %d\n", reg, value);
#endif

  if (reg >= 37)
    return;

  vdc[reg] = value;

  switch (reg)
    {
    case 1:			/* Columns displayed */
      return;

    case 6:			/* Rows displayed */
      return;

    case 9:			/* Scan Lines per Character */
      return;

    case 10:			/* Cursor Mode */
    case 11:
      return;

    case 12:			/* Display Address */
    case 13:
      v_vdc->screenmem = vdcRAM + (vdc[12] << 8) + vdc[13];
#if defined( VDC_REG_DEBUG )
      printf ("VDC screenmem = %d\n", v_vdc->screenmem - vdcRAM);
#endif
      return;

    case 14:			/* Cursor Address */
    case 15:
      return;

    case 18:			/* Video RAM Address -- RAM accessed via 31 */
    case 19:
      return;

    case 20:			/* Colour Address */
    case 21:
      v_vdc->colormem = vdcRAM + (vdc[20] << 8) + vdc[21];
#if defined( VDC_REG_DEBUG )
      printf ("VDC colormem = %d\n", v_vdc->colormem - vdcRAM);
#endif
      return;

    case 24:			/* Vertical Scroll */
      return;

    case 25:			/* Display Modes, Horisontal Scroll */
      return;

    case 26:			/* Display Colour */
      return;

    case 28:			/* Char Set Address */
      v_vdc->chargenmem = vdcRAM + ((vdc[28] & 0xe0) << 8);

#if defined( VDC_REG_DEBUG )
      printf ("VDC chargen = %d\n", v_vdc->chargenmem - vdcRAM);
#endif
      return;

    case 29:			/* Underline Scan Count */
      return;

    case 30:			/* Character Count (Copy/fill repetition) */
      /* vdc[0x12] = LOAD(0xa3d);
         vdc[0x13] = LOAD(0xa3c); */
      vdc_perform_fillcopy ();
      return;

    case 31:			/* Video RAM Data */
      vdc_perform_fillcopy ();
      return;

    case 32:			/* Block Copy Start Address */
    case 33:
      return;
    }

  /*else
     reg = value; */
  return;
}


/*

   read_vdc(): reads from one VDC register.

 */

BYTE
read_vdc (BYTE reg)
{
#if defined( VDC_REG_DEBUG )
  printf ("VDC read [%02X] %02x\n", reg, ((reg < 37) ? vdc[reg] : 0xff));
#endif

#if !defined( USE_VDC )
  return;
#endif

  if (reg == 31)
    {
      BYTE c = vdcRAM[(vdc[18] << 8) + vdc[19]];
      /* vdc_perform_fillcopy(); */
      return c;
    }
  return ((reg < 37) ? vdc[reg] : 0xff);
}

extern Window VDCborderwin;
static Window screenwin;


/*

   vdc_init(): VDC initialization sequence.
   
 */

int
vdc_init (void)
{
  vdc_set_vmode (screenwin, V_TEXT);
  return (0);
}


/*

   vdc_set_vmode(): sets the video mode according to the values in the registers (still useless
   now).

 */

void
vdc_set_vmode (Window win, int mode)
{
  if (debugflg)
    printf ("VDC vmode set to 0x%x\n", mode);
#if 0
  switch (mode)
    {
    case V_TEXT:
/*      XMapWindow(display, win); */
      break;

    case V_BITMAP:
/*      XMapWindow(display, win); */
      break;

    case V_BLANK:
/*      XUnmapWindow(display, win); */
      break;
    }

  if (mode == V_BITMAP)
    XUnmapWindow (display, win);
  else
    XMapWindow (display, win);
  vdc.mode = mode;
#endif
}


/* 

   vdc_perform_fillcopy() performs the copy or fill operation and updates the VDC regs
   consequently.
   
 */

void
vdc_perform_fillcopy (void)
{
  int ptr, ptr2;

  ptr2 = vdc[32] * 256 + vdc[33];
  ptr = vdc[18] * 256 + vdc[19];

  if (vdc[24] & 0x80)
    {
      /* memcpy( vdcRAM + ptr, vdcRAM + ptr2, vdc[ 30 ] ); */
      int i;

      for (i = 0; i < vdc[30]; i++)
	vdcRAM[ptr + i] = vdcRAM[ptr2 + i];
#if defined( VDC_REG_DEBUG )
      printf ("copying %d bytes from %d to %d\n", vdc[30] + 1, ptr2, ptr);
#endif
      ptr2 += vdc[30] + 1;
      vdc[32] = (ptr2 & 0xff00) >> 8;
      vdc[33] = ptr2 & 0xff;
    }
  else
    {
      memset (vdcRAM + ptr, vdc[31], vdc[30] + 1);
#if defined( VDC_REG_DEBUG )
      printf ("filling %d bytes with value %d from %d\n", vdc[30] + 1, byte, ptr);
#endif
    }
  ptr += vdc[30] + 1;
  vdc[19] = ptr & 0xff;
  vdc[18] = (ptr & 0xff00) >> 8;
  vdc[30] = 0;

#if defined( VDC_REG_DEBUG )
  printf ("(now)vdc18 = %d vdc19 = %d\n", vdc[18], vdc[19]);
#endif
}

/* Video Chip information */

/* This stuff should be useful when all the missing features will be implemented.

   #define VDC_BORD_WIDTH    vdc[2]
   #define VDC_SCR_COLS      vdc[0]*8
   #define VDC_XPIX          vdc[1]*8
   #define VDC_SCR_LINES     vdc[4]*8
   #define VDC_YPIX          vdc[6]*8
   #define VDC_BORD_HEIGHT   (YPIX-SCR_LINES)/2     

   
   #warning ALMOST_RANDOM_VALUES! -- should check these out!
   
   #define VDC_MAX_COLS      700    
   #define VDC_MAX_LINES     500
   #define VDC_BORD_WIDTH    0
   #define VDC_SCR_COLS      640
   #define VDC_SCR_LINES     200
   #define VDC_XPIX          640
   #define VDC_YPIX          200
   #define VDC_BORD_HEIGHT   0
 */

/* global */

#ifdef C128
extern BYTE mmu[];
#endif


/*

   VicII_Init(): Initializes the VIC-II emulation.

 */

Window
Vdc_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
	  char *res_name, int *w, int *h, char *icon_data[], Colormap * cmap_return,
	  int exact_speed, int asleep, int refrate)
{
  v_vdc = AllocVideo (SCR_COLS, SCR_LINES, BORD_WIDTH, BORD_HEIGHT, XPIX, YPIX, XSIZE, YSIZE,
		      VDC_NUMCOLORS, refrate, VDC_NUMSPRITES, exact_speed, win_name, asleep);
  if (!v_vdc)
    {
      fprintf (stderr, "fatal error: can't initialize Vdc emulation.\n");
      return -1;
    }
  if (OpenVideoWindow (disp, scr_num, v_vdc, win_name, win_name, res_class, res_name, icon_data,
		       vdc_cnames, cmap_return))
    {
      fprintf (stderr, "fatal error: can't open window for Vdc emulation.\n");
      return -1;
    }
  if (SetVideoModes (v_vdc, 1,
  /* VDC_NORMAL_MODE */
		     Vdc_GetTextData, Vdc_GetColorData,
		     NormalGetBgDataFromRegister, SPECIAL_VMODE,
		     Vdc_DrawLine))
    {
      fprintf (stderr, "fatal error: can't initialize Vdc emulation.\n");
      return -1;
    }
  v_vdc->vmode = 0;		/* VDC_NORMAL_MODE; */
  RefreshAll (v_vdc);
  *w = v_vdc->winwidth;
  *h = v_vdc->winheight;
  v_vdc->chargenmem = char_rom;
  v_vdc->screenmem = vdcRAM;
  v_vdc->colormem = vdcRAM + 2000;
  memset (v_vdc->screenmem, 0, 2000);
  memset (v_vdc->colormem, 1, 2000);
  return (v_vdc->win);
}


/*

   The following functions are used to read the screen data (see raster.c for more info)

 */

INLINE BYTE
Vdc_GetCharData (BYTE * coloraddr, BYTE * charaddr, int ll, BYTE * charmemlo, BYTE * charmemup,
		 BYTE flashpattern, BYTE curspattern, BYTE underlinepattern, BYTE * curspos)
{
  BYTE b;

  b = GET_CHAR_DATA ((*coloraddr) & VDC_ALTCHARSET_ATTR ? charmemlo : charmemup, *charaddr, ll);
  if (*coloraddr & VDC_UNDERLINE_ATTR)
    b |= underlinepattern;
  if (*coloraddr & VDC_REVERSE_ATTR)
    b = ~b;
  if (*coloraddr & VDC_FLASH_ATTR)
    b &= flashpattern;
  if (charaddr == curspos && curspattern)
    b = ~b;
  return b;
}

int
Vdc_GetTextData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  int i = 0, ll = l->n & 0x7, line = l->n >> 3;
  BYTE *curspos = vdcRAM + (vdc[14] << 8) + vdc[15];	/* cursor position */
  BYTE *coloraddr = v->colormem + line * v->tcols, *src = v->screenmem + line * v->tcols;
  BYTE *dest = l->fgdata, *charmemup = v->chargenmem, *charmemlo = v->chargenmem + 0x1000;
  BYTE curspattern, underlinepattern, flashpattern;

  curspattern = (vdc[10] & 0x60) >> 5;	/* bits 5-6 of register #10 specify cursor blink mode: */
  if (curspattern == 0x1)	/* 01: no cursor at all */
    curspattern = 0;
  else
    {
      if (ll >= (vdc[10] & 0x1f) && ll <= vdc[11])
	{
	  if (!curspattern)	/* 00: fixed cursor */
	    curspattern = 0xff;
	  else if (curspattern & 0x1)
	    /* 11: 1/32 blink */
	    curspattern = ((clk / CYCLES_PER_RFSH) & 0x20) ? 0xff : 0;
	  else
	    /* 10: 1/16 blink */
	    curspattern = ((clk / CYCLES_PER_RFSH) & 0x10) ? 0xff : 0;
	}
      else
	curspattern = 0;
    }
  /* register #29 specifies the line for underlining */
  underlinepattern = (ll == vdc[29] & 0x1f) ? 0xff : 0;
  flashpattern = ((clk / CYCLES_PER_RFSH) & 0x20) ? 0 : 0xff;

  if (no_check)
    {
      *xs = 0;
      *xe = v->tcols - 1;
      for (; i < v->tcols; i++, src++)
	dest[i] = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
				   curspattern, underlinepattern, curspos);
      return 1;
    }
  else
    {
      BYTE b;
      for (; i < v->tcols; i++, src++)
	{
	  b = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
			       curspattern, underlinepattern, curspos);
	  if (b != dest[i])
	    break;
	}
      if (i < v->tcols)
	{
	  *xs = *xe = i;
	  for (; i < v->tcols; i++, src++)
	    {
	      b = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
				   curspattern, underlinepattern, curspos);
	      if (dest[i] != b)
		{
		  dest[i] = b;
		  *xe = i;
		}
	    }
	  return 1;
	}
      else
	return 0;
    }
}

int
Vdc_GetColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  BYTE *addr = v->colormem + (l->n >> 3) * v->tcols;

  return (GetDataHiLo (l->colordata1, l->colordata2, addr, v->tcols, 1, xs, xe, no_check));
}

void
Vdc_DrawLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors)
{
  BYTE *colorp1, *colorp2, *pixelp;
  int i;

  colorp1 = &(l->colordata1[0]);
  colorp2 = &(l->colordata2[0]);
  pixelp = &(l->fgdata[0]);
  p += xs * CHAR_XSIZE;
  for (i = xs; i <= xe; i++, p += CHAR_XSIZE)
    {
      byte = pixelp[i];
      fg = colors[colorp2[i]];
      PUTMONO;
    }
}


/*

   Vdc_ArrangeWindow(): centers the image at the center of the window.

 */

void
Vdc_ArrangeWindow (int w, int h)
{
  ArrangeVideoImage (v_vdc, w, h);
  RefreshAll (v_vdc);
}

void
Vdc_Free (void)
{
  FreeVideo (v_vdc);
}

void
Vdc_Pause (void)
{
  PauseVideo (v_vdc);
}

void
Vdc_Restart (void)
{
  RestartVideo (v_vdc);
}

void
Vdc_DoRaster (CLOCK curr_clk)
{
  Raster (curr_clk, v_vdc);
}

/*

   The following is needed to turn Vdc emulation on and off in x128.

 */

void
Vdc_SetStatus (int awake)
{
  if ((v_vdc->asleep = !awake))
    XUnmapWindow (v_vdc->display, v_vdc->win);
  else
    XMapWindow (v_vdc->display, v_vdc->win);
  XSync (v_vdc->display, 1);
  SetSpeed (speed);
}
