
/*
 * $Id: vic2.c,v 1.11 1995/11/07 16:51:00 jopi Exp jopi $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Vesa-Matti Puro (vmp@lut.fi)
 *   Jarkko Sonninen (sonninen@lut.fi)
 *   Jouko Valta (jopi@stekt.oulu.fi)
 *
 *
 * $Log: vic2.c,v $
 * Revision 1.11  1995/11/07  16:51:00  jopi
 * *** empty log message ***
 *
 * Revision 1.10  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * VIC drawing routines renamed appropriately.
 * Recently introduced bug in Scroll-Redraw fixed.
 *
 * Revision 1.9  1994/12/12  16:59:44  jopi
 * New system
 *
 * Revision 1.8  1994/06/16  17:19:26  jopi
 * Code reorganized and cleaned up
 *
 * Revision 1.7  1994/06/08  12:21:08  jopi
 * Patchlevel 2
 *
 * Revision 1.6  1994/02/18  16:10:57  jopi
 * New debugger & arg parser and x128
 *
 * Revision 1.5  1994/01/26  16:08:37  jopi
 * X64 version 0.2 PL 1
 *
 * Revision 1.4  1993/11/10  01:55:34  jopi
 * reu, asm and disk directory fixed
 * REL_ADDR macro and 1541 made more portable
 *
 * Revision 1.3  93/06/21  13:39:47  jopi
 *  X64 version 0.2 PL 0
 *
 * Revision 1.2  1993/06/13  08:21:50  sonninen
 * *** empty log message ***
 *
 *
 */

#include <stdio.h>

#include "macro.h"
#include "vmachine.h"
#include "extern.h"


/* Video Chip information */

#define SCREENSIZE  1000

int win_xsize = XSIZE;
int win_ysize = YSIZE;
int win_xpix  = XPIX;
int win_ypix  = YPIX;
int screensize = SCREENSIZE;


/* global */

BYTE    vic[64];
BYTE    colorRam[0x400];

BYTE   *scrbase = NULL;		/* pointer to start of screen */
BYTE   *charbase = NULL;	/* pointer to start of chargen */
BYTE   *bitmap1_base = NULL;	/* pointer to upper half of bitmap */
BYTE   *bitmap2_base = NULL;	/* pointer to lower half of bitmap */

int    vbank=0;

int    rasterintline = 0;       /* line for raster IRQ */

#ifdef C128
extern BYTE mmu[];
#endif

int     yscroll = 0;
int     xscroll = 0;

static int  pow2[] = {
    0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80,
    0x100, 0x200, 0x400, 0x800, 0x1000, 0x2000, 0x4000, 0x8000
};


/*
 * Extern Functions
 */

BYTE   *calc_sprite_addr ( BYTE i );

/*
 * Local Variables
 */

static int videoint;	/* interrupt register for vic */
static CLOCK rasteru;	/* time when next raster count reaches raster
			 * register */

static int     charaddr;	/* chargen start address / 0x800 */
static ADDRESS scraddr;		/* screen start address */

/* ------------------------------------------------------------------------- */

void    init_vic(void)
{
    store_vic(0x18, 0x15);
}


/* this is called in the middle of init_memmaps() */
/*
void    set_video_hooks(void)
{
    int     bank;

    flush_queue();     ** Try to sync split pictures **

    switch (vmode) {
      case V_BLANK:
      case V_TILT:
	break;

      case V_TEXT:
      case V_ECTEXT:
      case V_MTEXT:
	** check if screen is drawn from ram ** 
	bank = vbank + ((vic[0x18] >> 6) & 3);
	if ((bank & 7) != 1) {
	    screenwr_hook = memwr[bank];
	    memwr[bank] = screenwr;
	    if (verflg)
		printf("set screen hook to bank %d\n", bank);
	}
	** chargen bank resolution is $800 (2k) **
	charaddr = (vbank << 1) + ((vic[0x18] >> 1) & 7);
	bank = charaddr >> 1;

	if ((bank & 7) != 1) {
	    matrix_hook1 = memwr[bank];
	    memwr[bank] = chargenwr;
	    if (verflg)
		printf("set chargen hook to bank %d chargen addr %x\n",
		    bank, charaddr * 0x800);
	}
	break;

      case V_BITMAP:
      case V_MBITMAP:
	** check if screen is drawn from ram ** 
	bank = vbank + ((vic[0x18] >> 6) & 3);
	if ((bank & 7) != 1) {
	    screenwr_hook = memwr[bank];
	    memwr[bank] = screenwr;
	    if (verflg)
		printf("set screen hook to bank %d\n", bank);
	}
	bank = vbank + ((vic[0x18] >> 2) & 2);
	matrix_hook1 = memwr[bank];
	memwr[bank] = matrixwr1;
	if(verflg)
	    printf("set matrix1 hook to bank %d\n", bank); **FCP**

	** chargen is visible $1000-$2000 and $9000-$a000 **
	bank |= 1;
	if ((bank & 7) != 1) {
	    matrix_hook2 = memwr[bank];
	    memwr[bank] = matrixwr2;
	    if(verflg)
		printf("set matrix2 hook to bank %d\n", bank); **FCP**
	}
	break;
    }
}

*/

void    screen_init(void)
{
    ADDRESS addr;
    BYTE   *tmp1, *tmp2;
    int     tmp3, tmp4, tmp5;

    /* you are not supposed to understand this ... */

    /*
     *  First calculate screen start address in memory
     */
    tmp1 = scrbase;
    scraddr = vbank + ( ( vic[0x18] & 0xf0 ) << 6 );
/*    scraddr = (vbank << 12) + ((vic[0x18] & 0xf0) << 6); */

    if ((scraddr & 0x7000) != 0x1000) {
	scrbase = ram + scraddr - vbank;
    } else {
	scrbase = char_rom + (scraddr & 0x800);
    }

    /* then update pointers for chargen and bitmap */
    tmp2 = charbase;
    addr = (vbank << 12) + ((vic[0x18] & 0xe) << 10);
    bitmap1_base = ram + (addr & 0xe000);

    if (debugflg)
	printf("charbase:%x\n", addr);

    if ((addr & 0x7000) != 0x1000) {
	charbase = ram + vbank + ((vic[0x18] & 0xe) << 10 );
	bitmap2_base = ram + ((addr & 0xf000) | 0x1000);
    } else {
	charbase = char_rom + (addr & 0x0800);
	bitmap2_base = char_rom;
    }
}

#define DRAW_SPRITE(i) 	x_move_sprite((i),\
		   vic[(i)<<1]+((vic[0x10]&pow2[i])?0x100:0),vic[((i)<<1)|1]);


/* ------------------------------------------------------------------------- */

void    store_vic(ADDRESS addr, BYTE byte)
{
    if ( addr != 0x12 && addr != 0x11 )
      vic[addr] = byte;
    else if ( addr == 0x11 )
      vic[addr] = vic[addr] & 0x80 | ( byte & 0x7f );

    if (debugflg)
	printf("store vic[%x] %x\n", addr, (int) byte);

    switch (addr) {

      case 17: /* $D011 */
	screen_init();
        rasterintline = ( rasterintline & 0xff ) | ( ( byte & 0x80 ) << 1 );
	update_raster(1);
	return;

      case 18: /* $D012  Raster Line Compare */
        rasterintline = ( rasterintline & 0x100 ) | byte;
	update_raster(1);
	return;

      case 24: /* $D018  Video and char matrix base address */
	screen_init();
	return;

 /* mmh.. better check this one... */

      case 25: /* $D019 IRQ flag register */
	return;

      case 26: /* $D01A IRQ mask register */
	/* light pen interrupt not implemented */
	if (byte & 8)
	    int_clk[I_LIGHT_PEN] = 0;
	else
	    int_clk[I_LIGHT_PEN] = 0;

	/* sprite to sprite collision not yet implemented */
	if (byte & 4)
	    int_clk[I_SS_COL] = 0;
	else
	    int_clk[I_SS_COL] = 0;

	/* sprite to background collision not yet implemented */
	if (byte & 2)
	    int_clk[I_SB_COL] = 0;
	else
	    int_clk[I_SB_COL] = 0;

	/* raster interrupts */
	if (byte & 1)
	    update_raster(1);
	else
	    int_clk[I_RASTER] = 0;
	find_next_int();
#ifdef NEW_TIMER_CODE
	update_intr_alarm();
#endif
	return;
#ifdef C128
      case 47: /* $D02F  Additional KBD rows */
	return;
      case 48: /* $D030  test, fast */
	return;
#endif
    }
}


BYTE    read_vic(ADDRESS addr)
{
    switch (addr) {

      case 17:
      case 18:
        return ( vic[addr] );

      case 19:
      case 20: /* Light Pen X,Y */
	return (0xff);

      case 25: /* $D019  Interrupt Register */
	{
	    BYTE    tmp;
	    if (clk > rasteru)
		update_raster(0);
	    tmp = videoint;
	    videoint = 0;
	    return (tmp);
	}

      case 26: /* $D01A  Interrupt Control */
	return (vic[addr]);

      case 30: /* $D01E  Sprite Sprite Collision */
	return (vic[addr]);

      case 31: /* $D01F  Sprite Background Collision */
	return (vic[addr]);

#ifdef C128
      case 47:/* $D02F  Additional KBD rows */
	return (vic[47] | 0xf8);
#endif

      default:
	return ((addr < 0x31) ? vic[addr] : 0xff);  /* VIC IIe size */
    }
}


void store_colorram(ADDRESS addr, BYTE byte)
{
    colorRam[addr] = byte;
}

BYTE read_colorram(ADDRESS addr)
{
  return colorRam[addr];
}

BYTE *calc_sprite_addr(BYTE i)

{
    ADDRESS addr = (vbank << 12) + (i << 6);

    if ((addr & 0x7000) != 0x1000) {
	return (ram + addr);
    } else {
	return (char_rom + (addr & 0xfff));
    }
}

void    int_raster(void)
{
    if (debugflg)
	printf("raster interrupt\n");
    update_raster(1);
    videoint |= 0x80 | 1;
}


void    update_raster(int flag)
{
    int     lines, currline;

    currline = ( vic[0x12] ) + ( ( vic[0x11] & 0x80 ) << 1 );
    if ( rasterintline > currline )
      lines = rasterintline - currline;
    else
      lines = 1 + SCR_LINES - currline + rasterintline; 
    if (lines < 0)
	printf("strange raster %d %d %d %ld %ld curr line %ld\n",
	    lines, (int) vic[0x12], (int) vic[0x11], rasteru, clk,
	    CURRENT_LINE(clk));
    if (rasteru < clk)
	videoint |= 1;
    rasteru = clk + lines * CYCLES_PER_LINE;
    if (flag && (vic[0x1a] & 1)) {
	int_clk[I_RASTER] = rasteru;
	find_next_int();
#ifdef NEW_TIMER_CODE
	update_intr_alarm();
#endif
    }
}
