/*
 * $Id: vicii.c,v 1.1 1996/04/01 05:49:29 ettore Exp $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * VIC-II video chip emulation.
 *
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vicii.c,v $
 * Revision 1.1  1996/04/01 05:49:29  ettore
 * Initial revision
 *
 */


#define _VICII_C

#include "extern.h"
#include "config.h"
#include "vicii.h"
#include "raster.c"
#include "vmachine.h"
#include "sprcoll.h"
#include "rasterutil.h"

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#if defined (C128)
#include "vdc.h"
#endif

/* #define VIC_II_VMODE_DEBUG */
/* #define VIC_II_RASTER_DEBUG */
/* #define VIC_II_INTERRUPT_DEBUG */

#if defined (C128) && defined (USE_VDC)
extern VideoStruct *v_vdc;
#endif

/* global */

BYTE vic[64];
BYTE colorRam[0x400];

BYTE *scrbase = colorRam;	/* pointer to start of screen */
BYTE *charbase = colorRam;	/* pointer to start of chargen */
BYTE *bitmapbase = colorRam;	/* pointer to start of bitmap */

int vbank = 0;
int rasterintline = 0;		/* line for raster IRQ */

#if defined (CHECK_SS_COLL)
int check_ss_coll = 1;
#else
int check_ss_coll = 1;
#endif

#if defined (CHECK_SB_COLL)
int check_sb_coll = 1;
#else
int check_sb_coll = 0;
#endif

VideoStruct *v_vic_ii;

#if defined (C128)
extern BYTE mmu[];
#endif

/* local Variables */

static int videoint;		/* interrupt register for vic */
static CLOCK rasteru;		/* time when next raster count reaches raster register */
static ADDRESS scraddr;		/* screen start address */

/* sprite collision masks */

static sprcollmask_t ss_collmask = 0;	/* sprite-sprite */
static sprcollmask_t sb_collmask = 0;	/* sprite-background */


/*

   screen_init(): initializes the VIC-II memory settings

 */

void
screen_init (void)
{
  ADDRESS addr;

  scraddr = vbank + ((vic[0x18] & 0xf0) << 6);

  if ((scraddr & 0x7000) != 0x1000)
    scrbase = ram + scraddr - vbank;
  else
    scrbase = char_rom + (scraddr & 0x800);

  addr = ((vic[0x18] & 0xe) << 10);
  bitmapbase = ram + (addr & 0xe000);
  addr += vbank;

  if (debugflg)
    printf ("charbase: %x\n", addr);

  if ((addr & 0x7000) != 0x1000)
    charbase = ram + addr;
  else
    charbase = char_rom + (addr & 0x0800);

  v_vic_ii->screenmem = scrbase + vbank;
  v_vic_ii->colormem = colorRam;
  v_vic_ii->bitmapmem = bitmapbase + vbank;
  v_vic_ii->chargenmem = charbase;
  v_vic_ii->vbankbase = ram + vbank;
  v_vic_ii->spriteptrbase = &v_vic_ii->screenmem[0x3f8];
}


/*

   The following functions are used to read the screen data (see raster.c for more info)

 */

int
VicII_GetHiresBitmapColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  if (l->n < 0 || l->n >= v->ypix)
    return GetDataHiLo (l->colordata1, l->bgdata, (BYTE *) "", v->tcols, 0, xs, xe, no_check);
  else
    {
      BYTE *addr = v->screenmem + (l->n >> 3) * v->tcols;
      return GetDataHiLo (l->colordata1, l->bgdata, addr, v->tcols, 1, xs, xe, no_check);
    }
}

int
VicII_GetHiresBitmapBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe)
{
  return 0;			/* no need to get background data (see prev. function) */
}

int
VicII_GetMCBitmapColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  int r;
  int step;
  BYTE *addr1, *addr2;

  if (l->n < 0 || l->n >= v->ypix)
    {
      step = 0;
      addr1 = addr2 = (BYTE *) "";
    }
  else
    {
      step = 1;
      addr1 = v->colormem + (l->n >> 3) * v->tcols;
      addr2 = v->screenmem + (l->n >> 3) * v->tcols;
    }
  r = GetDataHiLo (l->colordata1, l->colordata2, addr2, v->tcols, step, xs, xe, no_check);
  return GetData (l->colordata3, addr1, v->tcols, step, xs, xe, r);
}

int
VicII_GetMCTextColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  BYTE *addr = v->colormem + (l->n >> 3) * v->tcols;

  if (vic[0x22] != l->colordata1[0] || vic[0x23] != l->colordata2[0])
    {
      no_check = 1;
      *xs = 0;
      *xe = v->tcols - 1;
      l->colordata1[0] = vic[0x22];	/* we only use the first byte for MCTextMode */
      l->colordata2[0] = vic[0x23];
    }
  return GetData (l->colordata3, addr, v->tcols, 1, xs, xe, no_check);
}

int
VicII_GetExtTextData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  static BYTE fdata[40], bdata[40];
  BYTE b, *addr = v->screenmem + (l->n >> 3) * v->tcols;
  int i;

  for (i = 0; i < 40; i++)
    {
      fdata[i] = (b = addr[i]) & 0x3f;
      bdata[i] = b >> 6;
    }

  no_check = GetData (l->bgdata, bdata, v->tcols, 1, xs, xe, no_check);
  return GetCharData (l->fgdata, fdata, v->chargenmem, v->tcols, l->n & 0x7, xs, xe, no_check);
}

int
VicII_GetExtTextColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  int i;

  if (no_check || vic[0x21] != l->colordata2[0] || vic[0x22] != l->colordata2[1]
      || vic[0x23] != l->colordata2[2] || vic[0x24] != l->colordata2[3])
    {
      l->colordata2[0] = vic[0x21];
      l->colordata2[1] = vic[0x22];
      l->colordata2[2] = vic[0x23];
      l->colordata2[3] = vic[0x24];
      no_check = 1;
    }
  for (i = 0; i < v->tcols; i++)
    l->bgdata[i] = l->colordata2[l->bgdata[i]];
  return NormalGetColorData (v, l, xs, xe, no_check);
}

int
VicII_GetExtTextBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  return 0;			/* we already did this... */
}

int
VicII_GetBlackData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  return GetData (l->fgdata, (BYTE *) "", v->tcols, 0, xs, xe, no_check);
}

int
VicII_GetBlackColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  return GetDataHiLo (l->colordata1, l->bgdata, (BYTE *) "",  v->tcols, 0, xs, xe, no_check);
}

int
VicII_GetBlackBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  return 0;
}

void
VicII_DrawMCTextLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors)
{
  BYTE *colorp, *pixelp;
  int i;

  c1 = colors[vic[0x22] & 0xf];
  c2 = colors[vic[0x23] & 0xf];
  colorp = &(l->colordata3[0]);
  pixelp = &(l->fgdata[0]);
  p += xs * CHAR_XSIZE;
  for (i = xs; i <= xe; i++, p += CHAR_XSIZE)
    {
      byte = pixelp[i];
      if (colorp[i] & 0x8)
	{
	  c3 = colors[colorp[i] & 0x7];
	  PUTMULTI;
	}
      else
	{
	  fg = colors[colorp[i]];
	  PUTMONO;
	}
    }
}


/*

   VicII_SetVMode(): sets the video mode according to the values in the registers.

 */

void
VicII_SetVMode (void)
{
  if ((vic[0x11] & 0x40) && (vic[0x16] & 0x10))
    {
#if defined (VIC_II_VMODE_DEBUG)
      printf ("BLACK SCREEN");
#endif
      v_vic_ii->vmode = VIC_II_BLACK_MODE;
    }
  else
    {
      if (vic[0x11] & 0x20)
	if (vic[0x16] & 0x10)
	  {
	    v_vic_ii->vmode = VIC_II_MULTICOLOR_BITMAP_MODE;
#if defined (VIC_II_VMODE_DEBUG)
	    printf ("Multicolor Bitmap");
#endif
	  }
	else
	  {
	    v_vic_ii->vmode = VIC_II_HIRES_BITMAP_MODE;
#if defined (VIC_II_VMODE_DEBUG)
	    printf ("Hires Bitmap");
#endif
	  }
      else if (vic[0x11] & 0x40)
	{
	  v_vic_ii->vmode = VIC_II_EXTENDED_TEXT_MODE;
#if defined (VIC_II_VMODE_DEBUG)
	  printf ("Extended Text");
#endif
	}
      else
	{
	  if (vic[0x16] & 0x10)
	    {
	      v_vic_ii->vmode = VIC_II_MULTICOLOR_TEXT_MODE;
#if defined (VIC_II_VMODE_DEBUG)
	      printf ("Multicolor Text");
#endif
	    }
	  else
	    {
	      v_vic_ii->vmode = VIC_II_NORMAL_TEXT_MODE;
#if defined (VIC_II_VMODE_DEBUG)
	      printf ("Normal Text");
#endif
	    }
	}
#if defined (VIC_II_VMODE_DEBUG)
      printf (" Mode turned on at rasterline %d.\n", v_vic_ii->rasterline);
#endif
    }
}


/*

   store_vic(): stores a value in a VIC-II register.

 */

void
store_vic (ADDRESS addr, BYTE value)
{
  if (addr >= 0x20 && addr <= 0x2e)
    value &= 0xf;
  if (addr == 0x19)
    {
#if defined (VIC_II_INTERRUPT_DEBUG)
      printf ("store_vic $D019: videoint %x ->", videoint);
#endif
      videoint &= ~value;
#if defined (VIC_II_INTERRUPT_DEBUG)
      printf (" %x\n", videoint);
#endif
    }
  else
    {
      if (addr != 0x12 && addr != 0x11)
	vic[addr] = value;
      else if (addr == 0x11)
	vic[addr] = (vic[addr] & 0x80) | (value & 0x7f);
    }

  if (debugflg)
    printf ("store vic[%x] %x\n", addr, (int) value);

  if (addr < 16)
    {
      int sprnum = addr >> 1;
      if (addr & 0x1)
	v_vic_ii->sprites[sprnum].y = value;
      else
	{
	  v_vic_ii->sprites[sprnum].x &= 0x100;
	  v_vic_ii->sprites[sprnum].x |= value;
	}
    }
  else if (addr >= 39 && addr <= 46)
    v_vic_ii->sprites[addr - 39].color = value & 0xf;
  else
    {
      int i, b;

      switch (addr)
	{

	case 16:		/* $D010  Sprite X coord. MSB */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].x = (v_vic_ii->sprites[i].x & 0xff) |
	      (value & b ? 0x100 : 0);
	  return;

	case 17:		/* $D011 */
	  rasterintline = (rasterintline & 0xff) | ((value & 0x80) << 1);
	  update_raster (1);
	  v_vic_ii->ysmooth = (value & 0x7) - 3;
	  v_vic_ii->blank = !(value & 0x10);
	  if (value & 0x8)
	    v_vic_ii->brdgrow_yu = v_vic_ii->brdgrow_yl = 0;
	  else
	    /* ``open vertical borders'' effect */
	    {
	      if (!v_vic_ii->brdgrow_yu)
		{
		  int k = v_vic_ii->bordheight + v_vic_ii->ypix - v_vic_ii->rasterline;
		  if (k >= 0 && k <= 4)
		    v_vic_ii->openvborders = 1;
		}
	      v_vic_ii->brdgrow_yu = v_vic_ii->brdgrow_yl = 4;
	    }
	  VicII_SetVMode ();
	  return;

	case 18:		/* $D012  Raster Line Compare */
	  rasterintline = (rasterintline & 0x100) | value;
	  update_raster (1);
	  return;

	case 21:		/* $D015  Sprite Enable */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].viewable = value & b ? 1 : 0;
	  return;

	case 22:		/* $D016 */
	  v_vic_ii->xsmooth = value & 0x7;
	  if (value & 0x8)
	    v_vic_ii->brdgrow_xr = v_vic_ii->brdgrow_xl = 0;
	  else
	    {
	      v_vic_ii->brdgrow_xr = 9;
	      v_vic_ii->brdgrow_xl = 7;
	    }
	  VicII_SetVMode ();
	  return;

	case 23:		/* $D017  Sprite Y-expand */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].yexp = value & b ? 1 : 0;
	  return;

	case 24:		/* $D018  Video and char matrix base address */
	  screen_init ();
	  return;

	case 25:		/* $D019 IRQ flag register */
	  return;

	case 26:		/* $D01A IRQ mask register */
	  /* light pen interrupt not implemented */
	  if (value & 8)
	    int_clk[I_LIGHT_PEN] = 0;
	  else
	    int_clk[I_LIGHT_PEN] = 0;

#if 0
	  /* sprite to sprite collision not yet implemented */
	  if (value & 4)
	    int_clk[I_SS_COL] = 0;
	  else
	    int_clk[I_SS_COL] = 0;

	  /* sprite to background collision not yet implemented */
	  if (value & 2)
	    int_clk[I_SB_COL] = 0;
	  else
	    int_clk[I_SB_COL] = 0;
#endif /* now they _are_ implemented!!! EP 03/09/96 */

	  /* raster interrupts */
	  if (value & 1)
	    update_raster (1);
	  else
	    int_clk[I_RASTER] = 0;
	  find_next_int ();
#if defined (NEW_TIMER_CODE)
	  update_intr_alarm ();
#endif
	  return;

	case 27:		/* $D01B  Sprite Priority */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].priority = value & b ? 1 : 0;
	  return;

	case 28:		/* $D01C  Sprite Multicolor select */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].multicolor = value & b ? 1 : 0;
	  return;

	case 29:		/* $D01D  Sprite X-expand */
	  for (i = 0, b = 0x01; i < 8; b <<= 1, i++)
	    v_vic_ii->sprites[i].xexp = value & b ? 1 : 0;
	  return;

	case 32:		/* $D020  Border color */
	  v_vic_ii->bordcol = value & 0xf;
	  return;

	case 33:		/* $D021  Background color */
	  v_vic_ii->bgcol = value & 0xf;
	  return;

	case 37:		/* $D025  Sprite multicolor register #1 */
	  v_vic_ii->sprmc1 = value & 0xf;
	  return;

	case 38:		/* $D026  Sprite multicolor register #2 */
	  v_vic_ii->sprmc2 = value & 0xf;
	  return;

#if defined (C128)
	case 47:		/* $D02F  Additional KBD rows */
	  return;
	case 48:		/* $D030  test, fast */
	  return;
#endif
	}
    }
}


/*

   read_vic(): reads from one VIC-II register.

 */

BYTE
read_vic (ADDRESS addr)
{
  switch (addr)
    {

    case 17:
#if defined (VIC_II_RASTER_DEBUG)
      printf ("read raster (highest bit): rasterline = %d\n", v_vic_ii->rasterline);
#endif
      return ((vic[addr] & 0x7f) |
	      (((v_vic_ii->rasterline) & 0x100) >> 1));
    case 18:
#if defined (VIC_II_RASTER_DEBUG)
      printf ("read raster (lowest bits): rasterline = %d\n", v_vic_ii->rasterline);
#endif
      return ((v_vic_ii->rasterline) & 0xff);

    case 19:
    case 20:			/* $D013, $D014  Light Pen X,Y */
      return 0xff;

    case 25:			/* $D019  Interrupt Register */
      {
	BYTE tmp;
	if (clk > rasteru)
	  update_raster (0);
	tmp = videoint;
#if 0
	videoint = 0;
#endif
#if defined (VIC_II_INTERRUPT_DEBUG)
	printf ("read VIC interrupt register: %x\n", tmp);
#endif
	return tmp;
      }

    case 26:			/* $D01A  Interrupt Control */
      return vic[addr];

    case 30:			/* $D01E  Sprite Sprite Collision */
      vic[addr] = ss_collmask;
      ss_collmask = 0;
#if defined (DEBUG_SS_COLLISIONS)
      printf ("ss_collmask reset by $D01E read at line 0x%x.\n", v_vic_ii->rasterline);
#endif
      return vic[addr];

    case 31:			/* $D01F  Sprite Background Collision */
      vic[addr] = sb_collmask;
      sb_collmask = 0;
#if defined (DEBUG_SB_COLLISIONS)
      printf ("sb_collmask reset by $D01F read at line 0x%x.\n", v_vic_ii->rasterline);
#endif
      return vic[addr];

#if defined (C128)
    case 47:			/* $D02F  Additional KBD rows */
      return (vic[47] | 0xf8);
#endif

    default:
      return ((addr < 0x31) ? vic[addr] : 0xff);	/* VIC IIe size */
    }
}


/*

   store_colorram(): stores a value in the VIC-II color RAM.

 */

void
store_colorram (ADDRESS addr, BYTE value)
{
  colorRam[addr] = value & 0xf;
}


/*

   read_colorram(): reads a value from the VIC-II color RAM.

 */

BYTE
read_colorram (ADDRESS addr)
{
  return colorRam[addr];
}


/*

   int_raster(): VIC-II raster interrupt.

 */

void
int_raster (void)
{
#if !defined (VIC_II_INTERRUPT_DEBUG)
  if (debugflg)
#endif
    printf ("raster interrupt\n");
  update_raster (1);
  videoint |= 0x80 | 1;
}


/*

   update_raster(): updates the raster variable so that the next interrupt is generated correctly.

 */

void
update_raster (int flag)
{
  int lines;

  videoint = 0;
  if (rasterintline > v_vic_ii->rasterline)
    lines = rasterintline - v_vic_ii->rasterline;
  else
    lines = v_vic_ii->scrlines - v_vic_ii->rasterline + rasterintline;
  if (lines < 0)
    printf ("strange raster %d %d %d %ld %ld curr line %ld\n",
	    lines, (int) vic[0x12], (int) vic[0x11], rasteru, clk,
	    CURRENT_LINE (clk));
  if (rasteru < clk)
    videoint |= 1;
  rasteru = (clk / CYCLES_PER_LINE) * CYCLES_PER_LINE + lines * CYCLES_PER_LINE;
  if (flag && (vic[0x1a] & 1))
    {
      int_clk[I_RASTER] = rasteru;
      find_next_int ();
#if defined (NEW_TIMER_CODE)
      update_intr_alarm ();
#endif
    }
#if defined (VIC_II_RASTER_DEBUG) || defined (VIC_II_INTERRUPT_DEBUG)
  printf ("update_raster(%d): lines = %d, rasteru = %ld, vic[0x1a] & 1 = %d\n",
	  flag, lines, rasteru, vic[0x1a] & 1);
#endif
}


/*

   VicII_Init(): Initializes the VIC-II emulation.

 */

Window
VicII_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
	    char *res_name, int *width, int *height, char *icon_data[], Colormap * cmap_return,
	    int exact_speed, int asleep, int refrate)
{
  v_vic_ii = AllocVideo (SCR_COLS, SCR_LINES, BORD_WIDTH, BORD_HEIGHT,
			 XPIX, YPIX, XSIZE, YSIZE, VIC_II_NUMCOLORS, refrate,
			 VIC_II_NUM_SPRITES, exact_speed, win_name, asleep);
  if (!v_vic_ii)
    {
      fprintf (stderr, "fatal error: can't initialize Vic-II emulation.\n");
      return -1;
    }
  if (OpenVideoWindow (disp, scr_num, v_vic_ii, win_name, win_name, res_class, res_name,
		       icon_data, vic_ii_cnames, cmap_return))
    {
      fprintf (stderr, "fatal error: can't open window for Vic-II emulation.\n");
      return -1;
    }
  if (SetVideoModes (v_vic_ii, 6,
  /* VIC_II_NORMAL_TEXT_MODE */
		     NormalGetTextData, NormalGetColorData,
		     NormalGetBgDataFromRegister, HIRES_VMODE,
  /* VIC_II_MULTICOLOR_TEXT_MODE */
		     NormalGetTextData, VicII_GetMCTextColorData,
		     NormalGetBgDataFromRegister, SPECIAL_VMODE, VicII_DrawMCTextLine,
  /* VIC_II_EXTENDED_TEXT_MODE */
		     VicII_GetExtTextData, VicII_GetExtTextColorData,
		     VicII_GetExtTextBgData, HIRES_VMODE,
  /* VIC_II_HIRES_BITMAP_MODE */
		     NormalGetBitmapData, VicII_GetHiresBitmapColorData,
		     VicII_GetHiresBitmapBgData, HIRES_VMODE,
  /* VIC_II_MULTICOLOR_BITMAP_MODE */
		     NormalGetBitmapData, VicII_GetMCBitmapColorData,
		     NormalGetBgDataFromRegister, MULTICOLOR_VMODE,
  /* VIC_II_BLACK_MODE */
		     VicII_GetBlackData, VicII_GetBlackColorData,
		     VicII_GetBlackBgData, HIRES_VMODE))
    {
      fprintf (stderr, "fatal error: can't initialize Vic-II emulation.\n");
      return -1;
    }
  v_vic_ii->vmode = VIC_II_NORMAL_TEXT_MODE;
  store_vic (0x18, 0x15);
  VicII_SetVMode ();
  screen_init ();
  RefreshAll (v_vic_ii);
  *width = v_vic_ii->winwidth;
  *height = v_vic_ii->winheight;
  return v_vic_ii->win;
}


/*

   VicII_ArrangeWindow(): centers the image at the center of the window.

 */

void
VicII_ArrangeWindow (int width, int height)
{
  ArrangeVideoImage (v_vic_ii, width, height);
  RefreshAll (v_vic_ii);
}


void
VicII_Free (void)
{
  FreeVideo (v_vic_ii);
}


void
VicII_Pause (void)
{
  PauseVideo (v_vic_ii);
}


void
VicII_Restart (void)
{
  RestartVideo (v_vic_ii);
}


/*

   This one performs the collision checking too...

 */

void
DoRaster (CLOCK curr_clk)
{
  static sprmask_struct sprmasks[8];
  int line;

  if ((line = Raster (curr_clk, v_vic_ii)) != -1)
    if (check_ss_coll || check_sb_coll)
      {
	int nmasks = build_sprite_masks (v_vic_ii, sprmasks, line);
	if (nmasks > 2 && check_ss_coll)
	  /* check sprite-sprite collisions */
	  {
	    ss_collmask |= check_ss_collisions (sprmasks);
	    if (ss_collmask)
	      {
		if (vic[0x1a] & 4)
		  int_clk[I_SS_COL] = 1;
#if defined (DEBUG_SS_COLLISIONS)
		printf ("ss collision! mask = 0x%x ", ss_collmask);
		print_binary (ss_collmask);
		printf (" at line %d\n", v_vic_ii->rasterline);
#endif
	      }
	  }
	if (nmasks && check_sb_coll)
	  /* check sprite-background collisions */
	  {
	    sb_collmask |= check_sb_collisions (v_vic_ii, sprmasks, line);
	    if (sb_collmask)
	      {
		if (vic[0x1a] & 2)
		  int_clk[I_SB_COL] = 1;
#if defined (DEBUG_SB_COLLISIONS)
		printf ("sb collision! mask = 0x%x ", sb_collmask);
		print_binary (sb_collmask);
		printf (" at line %d\n", v_vic_ii->rasterline);
#endif
	      }
	  }
      }
#if defined (C128) && defined (USE_VDC)
  Vdc_DoRaster (curr_clk);
#endif
}


/*

   The following is needed to turn Vic-II emulation on and off in x128.

 */

void
VicII_SetStatus (int awake)
{
  if ((v_vic_ii->asleep = !awake))
    XUnmapWindow (v_vic_ii->display, v_vic_ii->win);
  else
    XMapWindow (v_vic_ii->display, v_vic_ii->win);
  XSync (v_vic_ii->display, 1);
  SetSpeed (speed);
}
